<?php
/**
 * @version 2.0 2012-01-20
 * @package Joomla
 * @subpackage Intellectual Property
 * @copyright (C) 2012 the StarTech
 * @license GNU/GPL see LICENSE.php
 */

// no direct access
defined( '_JEXEC' ) or die( 'Restricted access' );
$app->registerEvent( 'onSearch', 'plgSearchJFIproperty' );
JPlugin::loadLanguage( 'plg_search_jfiproperty' );

class plgSearchJFIproperty extends JPlugin
{
	function onSearchAreas()
	{
		static $areas = array(
			'iproperty' => 'Properties'
		);
		return $areas;
	}

    function onSearch( $text, $phrase='', $ordering='', $areas=null )
	{
		global $app;

        $db		= JFactory::getDBO();
        $user	= JFactory::getUser();
        $groups	= $user->getAuthorisedViewLevels();

        $registry = JFactory::getConfig();
        $lang = $registry->getValue("config.jflang");

        require_once(JPATH_SITE.DS.'administrator'.DS.'components'.DS.'com_search'.DS.'helpers'.DS.'search.php');
        require_once(JPATH_SITE.DS.'administrator'.DS.'components'.DS.'com_iproperty'.DS.'classes'.DS.'admin.class.php');
        require_once(JPATH_SITE.DS.'components'.DS.'com_iproperty'.DS.'helpers'.DS.'html.helper.php');
        require_once(JPATH_SITE.DS.'components'.DS.'com_iproperty'.DS.'helpers'.DS.'route.php');
        $settings = ipropertyAdmin::config();

        $searchText = $text;
		if (is_array( $areas )) {
			if (!array_intersect( $areas, array_keys( $this->onSearchAreas() ) )) {
				return array();
			}
		}

        // load plugin params info
        $plugin = JPluginHelper::getPlugin('search', 'jfiproperty');
        $pluginParams = new JParameter( $plugin->params );

        $limit          = $pluginParams->def( 'search_limit', 50 );
        $activeLang 	= $pluginParams->def( 'active_language_only', 0);

        $text = trim( $text );
        if ($text == '') {
            return array();
        }

        $wheres 	= array();
        switch ($phrase)
        {
            case 'exact':
                $text		= $db->Quote( '%'.$db->getEscaped( $text, true ).'%', false );
                $where = "LOWER(jfc.value) LIKE ".$text;
                break;

            case 'all':
            case 'any':
            default:
                $words 	= explode( ' ', $text );
                $wheres = array();
                foreach ($words as $word) {
                    $word		= $db->Quote( '%'.$db->getEscaped( $word, true ).'%', false );
                    $wheres[] 	= "LOWER(jfc.value) LIKE ".$word;
                }
                $where = '(' . implode( ($phrase == 'all' ? ') AND (' : ') OR ('), $wheres ) . ')';
                break;
        }

        switch ($ordering) {
			case 'oldest':
				$order = 'p.created ASC';
				break;

			case 'popular':
				$order = 'p.hits DESC';
				break;

            case 'alpha':
                $order = ($settings->showtitle) ? 'p.title ASC' : 'p.street ASC';
				break;

			case 'category':
				$order = ($settings->showtitle) ? 'pm.cat_id ASC, p.title ASC' : 'pm.cat_id ASC, p.street ASC';

			case 'newest':
				default:
				$order = 'p.created DESC';
				break;
		}

        if ( $limit > 0 )
		{
            // Filter by start and end dates.
            $nullDate   = $db->Quote($db->getNullDate());
            $date       = JFactory::getDate();
            $nowDate    = $db->Quote($date->toSql());
            
            $query = $db->getQuery(true);
            $query->select('p.title, p.mls_id, p.street_num, p.street, p.street2, p.created AS created, p.alias as alias, p.hide_address AS hidden, p.short_description, p.description, c.title AS section, p.id AS property_id, c.id AS cat_id, p.hits, "2" AS browsernav')
                    ->from('#__iproperty as p')
                    ->innerJoin('#__iproperty_propmid as pm on pm.prop_id = p.id')
                    ->innerJoin('#__iproperty_categories as c on c.id = pm.cat_id')
                    ->leftJoin('#__jf_content as jfc on reference_id = p.id')
                    ->leftJoin('#__languages as jfl on jfl.id = jfc.language_id')
                    ->where('p.state = 1 and p.approved = 1 and c.state = 1')
                    ->where('(p.publish_up = '.$nullDate.' OR p.publish_up <= '.$nowDate.')')
                    ->where('(p.publish_down = '.$nullDate.' OR p.publish_down >= '.$nowDate.')')
                    ->where('(c.publish_up = '.$nullDate.' OR c.publish_up <= '.$nowDate.')')
                    ->where('(c.publish_down = '.$nullDate.' OR c.publish_down >= '.$nowDate.')')
                    ->where('jfc.reference_table = "iproperty"');
            if($activeLang){
                $query->where('jfl.code = '.$lang);
            }
            if(is_array($groups) && !empty($groups)){
                $query->where('p.access IN ('.implode(",", $groups).')')
                    ->where('c.access IN ('.implode(",", $groups).')');
            }
            $query->group('p.id');
            $query->order($order);

			$db->setQuery( $query, 0, $limit );
			$list = $db->loadObjectList();
			$limit -= count($list);
            //echo $query;

			if(isset($list))
			{
				foreach($list as $key => $item)
				{
					$available_cats = ipropertyHTML::getAvailableCats($item->property_id);
                    $first_cat = $available_cats[0];
                    $list[$key]->title = ($settings->showtitle && $item->title) ? $item->section.': '.$item->title : ((!$settings->street_num_pos) ? $item->section.': '.$item->street_num.' '.$item->street.' '.$item->street2 : $item->section.': '.$item->street.' '.$item->street2.' '.$item->street_num);
                    $list[$key]->href = JRoute::_(ipropertyHelperRoute::getPropertyRoute($item->property_id.':'.$item->alias, $first_cat));
                    //$list[$key]->href = JRoute::_( JURI::root() . 'index.php?option=com_iproperty&view=property&id=' . $item->property_id );
				}
			}
			$rows[] = $list;
		}

        $results = array();
		if(count($rows))
		{
			foreach($rows as $row)
			{
				$new_row = array();
				foreach($row AS $key => $post) {
					if(searchHelper::checkNoHTML($post, $searchText, array('text', 'title'))) {
						$new_row[] = $post;
					}
				}
				$results = array_merge($results, (array) $new_row);
			}
		}

		return $results;
    }
}