<?php
/**
 * @version 2.0 2012-01-20
 * @package Joomla
 * @subpackage Intellectual Property
 * @copyright (C) 2012 the StarTech
 * @license GNU/GPL see LICENSE.php
 */

defined( '_JEXEC' ) or die( 'Restricted access' );

class IpropertyHelperQuery extends JObject
{
    var $_sort  = null;
    var $_order = null;
    var $_db    = null;

    function setOrderBy($_sort, $_order)
    {
        $this->_sort = $_sort;
        $this->_order = $_order;
    }

    function getOrderBy()
    {
        if(!isset($this->_sort) || !isset($this->_order)) {
            $this->_sort = 'p.street';
            $this->_order = 'ASC';
        }
        return ' ORDER BY ' . $this->_sort . ' ' . $this->_order;
    }

    function buildPropertiesQuery( $where, $debug = null )
    {
        $user       = JFactory::getUser();
        $groups     = $user->getAuthorisedViewLevels();
        
        // Filter by start and end dates.
        $nullDate   = $this->_db->Quote($this->_db->getNullDate());
        $date       = JFactory::getDate();
        $nowDate    = $this->_db->Quote($date->toSql());
        
        $query = $this->_db->getQuery(true);
        $query->select('p.*, p.id AS id, p.title AS title, p.street_num, p.street, p.street2, p.description as description, p.created as fcreated, co.name as company_name, p.alias as prop_alias, c.alias as cat_alias, a.alias as agent_alias, co.alias as co_alias')
            ->from('#__iproperty AS p')
            ->leftJoin('#__iproperty_propmid as pm on pm.prop_id = p.id')
            ->leftJoin('#__iproperty_agentmid as am on am.prop_id = p.id')
            ->leftJoin('#__iproperty_categories as c on c.id = pm.cat_id')
            ->leftJoin('#__iproperty_companies as co on co.id = p.listing_office')
            ->leftJoin('#__iproperty_agents as a on a.id = am.agent_id')        
            ->where('(p.publish_up = '.$nullDate.' OR p.publish_up <= '.$nowDate.')')
            ->where('(p.publish_down = '.$nullDate.' OR p.publish_down >= '.$nowDate.')')
            ->where('(c.publish_up = '.$nullDate.' OR c.publish_up <= '.$nowDate.')')
            ->where('(c.publish_down = '.$nullDate.' OR c.publish_down >= '.$nowDate.')')
            ->where('p.state = 1 AND c.state = 1 AND a.state = 1 AND co.state = 1')
            ->where('p.approved = 1');
        if(is_array($groups) && !empty($groups)){
            $query->where('p.access IN ('.implode(",", $groups).')')
                ->where('c.access IN ('.implode(",", $groups).')');
        }
        if( !empty($where) ) {
            if(is_array($where)) {
                foreach($where as $w){
                    $query->where($w);
                }
            } else {
                $query->where($where);
            }
        }
        $query->group('p.id');
        if(!isset($this->_sort) || !isset($this->_order)) {
            $query->order('p.street ASC');
        } else {
            $query->order($this->_sort.' '.$this->_order);
        }

        if($debug == 1) echo $query . '<br /><br />';
        return $query;
    }
    
    function buildIPUserPropertiesQuery( $where, $debug = null )
    {
        $user       = JFactory::getUser();
        $groups     = $user->getAuthorisedViewLevels();

        // Filter by start and end dates.
        $nullDate   = $this->_db->Quote($this->_db->getNullDate());
        $date       = JFactory::getDate();
        $nowDate    = $this->_db->Quote($date->toSql());
        
        $query = $this->_db->getQuery(true);
        $query->select('p.mls_id, p.id AS id, p.price, p.title AS title, p.street_num, p.street, p.street2, p.city, p.locstate, p.state, p.featured, p.approved, p.created as fcreated, p.alias as alias')
            ->from('#__iproperty as p')
            ->join('', '#__iproperty_agentmid as am ON am.prop_id = p.id');
        if(is_array($groups) && !empty($groups)){
            $query->where('p.access IN ('.implode(",", $groups).')');
        }
        if( !empty($where) ) {
            if(is_array($where)) {
                foreach($where as $w){
                    $query->where($w);
                }
            } else {
                $query->where($where);
            }
        }
        $query->group('p.id');
        if(!isset($this->_sort) || !isset($this->_order)) {
            $query->order('p.id ASC');
        } else {
            $query->order($this->_sort.' '.$this->_order);
        }

        if($debug == 1) echo $query . '<br /><br />';
        return $query;
    }    

    function buildAdvPropertiesQuery( $where, $debug = null )
    {
        $user       = JFactory::getUser();
        $groups     = $user->getAuthorisedViewLevels();

        // Filter by start and end dates.
        $nullDate   = $this->_db->Quote($this->_db->getNullDate());
        $date       = JFactory::getDate();
        $nowDate    = $this->_db->Quote($date->toSql());
        
        $query = $this->_db->getQuery(true);
        $query->select('p.id AS id, p.title AS title, p.street_num, p.street, p.street2, p.apt, p.city, p.short_description as short_description, p.mls_id, p.stype_freq, p.latitude, p.longitude, p.price, p.beds, p.baths, p.sqft, p.call_for_price, p.created, p.modified, p.stype, p.hide_address, LEFT(p.description,300) as description, p.alias as prop_alias')
            ->from('#__iproperty AS p')
            ->leftJoin('#__iproperty_propmid as pm on pm.prop_id = p.id')
            ->leftJoin('#__iproperty_agentmid as am on am.prop_id = p.id')
            ->leftJoin('#__iproperty_categories as c on c.id = pm.cat_id')
            ->leftJoin('#__iproperty_companies as co on co.id = p.listing_office')
            ->leftJoin('#__iproperty_agents as a on a.id = am.agent_id')        
            ->where('(p.publish_up = '.$nullDate.' OR p.publish_up <= '.$nowDate.')')
            ->where('(p.publish_down = '.$nullDate.' OR p.publish_down >= '.$nowDate.')')
            ->where('(c.publish_up = '.$nullDate.' OR c.publish_up <= '.$nowDate.')')
            ->where('(c.publish_down = '.$nullDate.' OR c.publish_down >= '.$nowDate.')')
            ->where('p.state = 1 AND c.state = 1 AND a.state = 1 AND co.state = 1')
            ->where('p.approved = 1');
        if(is_array($groups) && !empty($groups)){
            $query->where('p.access IN ('.implode(",", $groups).')')
                ->where('c.access IN ('.implode(",", $groups).')');
        }
        if( !empty($where) ) {
            if(is_array($where)) {
                foreach($where as $w){
                    $query->where($w);
                }
            } else {
                $query->where($where);
            }
        }
        $query->group('p.id');
        if(!isset($this->_sort) || !isset($this->_order)) {
            $query->order('p.street ASC');
        } else {
            $query->order($this->_sort.' '.$this->_order);
        }

        if($debug == 1) echo $query . '<br /><br />';
        return $query;
    }

    function buildOpenhouseQuery( $where, $debug = null )
    {
        $user       = JFactory::getUser();
        $groups     = $user->getAuthorisedViewLevels();

        // Filter by start and end dates.
        $nullDate   = $this->_db->Quote($this->_db->getNullDate());
        $date       = JFactory::getDate();
        $nowDate    = $this->_db->Quote($date->toSql());

        $query = $this->_db->getQuery(true);
        $query->select('p.*, p.id AS id, p.title AS title, p.street_num, p.street, p.street2, p.description as description, oh.name as ohname, oh.openhouse_start as ohstart, oh.openhouse_end as ohend, oh.comments as comments, p.created as fcreated, oh.openhouse_start as startdate, oh.openhouse_end as enddate')
            ->from('#__iproperty AS p')
            ->leftJoin('#__iproperty_openhouses as oh ON oh.prop_id = p.id')
            ->leftJoin('#__iproperty_propmid as pm on pm.prop_id = p.id')
            ->leftJoin('#__iproperty_agentmid as am on am.prop_id = p.id')
            ->leftJoin('#__iproperty_categories as c on c.id = pm.cat_id')
            ->leftJoin('#__iproperty_companies as co on co.id = p.listing_office')
            ->leftJoin('#__iproperty_agents as a on a.id = am.agent_id')        
            ->where('(p.publish_up = '.$nullDate.' OR p.publish_up <= '.$nowDate.')')
            ->where('(p.publish_down = '.$nullDate.' OR p.publish_down >= '.$nowDate.')')
            ->where('(c.publish_up = '.$nullDate.' OR c.publish_up <= '.$nowDate.')')
            ->where('(c.publish_down = '.$nullDate.' OR c.publish_down >= '.$nowDate.')')
            ->where('p.state = 1 AND c.state = 1 AND a.state = 1 AND co.state = 1 AND oh.state = 1')
            ->where('oh.openhouse_end >= '.$nowDate)
            ->where('p.approved = 1');
        if(is_array($groups) && !empty($groups)){
            $query->where('p.access IN ('.implode(",", $groups).')')
                ->where('c.access IN ('.implode(",", $groups).')');
        }
        if( !empty($where) ) {
            if(is_array($where)) {
                foreach($where as $w){
                    $query->where($w);
                }
            } else {
                $query->where($where);
            }
        }
        $query->group('p.id, oh.openhouse_end');
        if(!isset($this->_sort) || !isset($this->_order)) {
            $query->order('oh.openhouse_end ASC');
        } else {
            $query->order($this->_sort.' '.$this->_order);
        }

        if($debug == 1) echo $query . '<br /><br />';
        return $query;
    }

    function buildAgentsQuery( $where, $debug = null )
    {        
        $query = $this->_db->getQuery(true);
        
        $query->select('a.*, c.id AS companyid, c.name AS companyname, CONCAT_WS(" ",fname,lname) AS name, c.alias as co_alias')
            ->from('#__iproperty_agents as a')
            ->leftJoin('#__iproperty_companies as c on c.id = a.company')
            ->where('a.state = 1 AND c.state = 1');
        if( !empty($where) ) {
            if(is_array($where)) {
                foreach($where as $w){
                    $query->where($w);
                }
            } else {
                $query->where($where);
            }
        }
        $query->group('a.id');
        if(!isset($this->_sort) || !isset($this->_order)) {
            $query->order('a.ordering ASC');
        } else {
            $query->order($this->_sort.' '.$this->_order);
        }

        if($debug == 1) echo $query . '<br /><br />';
        return $query;
    }
    
    function buildIPUserAgentsQuery( $where, $debug = null )
    {        
        $query = $this->_db->getQuery(true);
        
        $query->select('a.*, c.id AS companyid, c.name AS companyname, CONCAT_WS(" ",fname,lname) AS name, a.alias as alias')
            ->from('#__iproperty_agents as a')
            ->leftJoin('#__iproperty_companies as c on c.id = a.company');
        if( !empty($where) ) {
            if(is_array($where)) {
                foreach($where as $w){
                    $query->where($w);
                }
            } else {
                $query->where($where);
            }
        }
        $query->group('a.id');
        if(!isset($this->_sort) || !isset($this->_order)) {
            $query->order('a.ordering ASC');
        } else {
            $query->order($this->_sort.' '.$this->_order);
        }

        if($debug == 1) echo $query . '<br /><br />';
        return $query;
    }

    function incrementPropertyHits($propid = '')
    {
        if($propid){
            $db = JFactory::getDbo();
            $query = $db->getQuery(true);
            $query->update('#__iproperty')
                ->set('hits=(hits+1)')
                ->where('id = '.(int)$propid);
            
            return $query;
        }
    }

    function getCategories($parent = '', $order = 'ordering ASC')
    {
        $db         = JFactory::getDbo();
        $user       = JFactory::getUser();
        $groups     = $user->getAuthorisedViewLevels();        
        
        // Filter by start and end dates.
        $nullDate   = $db->Quote($db->getNullDate());
        $date       = JFactory::getDate();
        $nowDate    = $db->Quote($date->toSql());
        
        $query = $db->getQuery(true);
        $query->select('*')
            ->from('#__iproperty_categories')
            ->where('(publish_up = '.$nullDate.' OR publish_up <= '.$nowDate.')')
            ->where('(publish_down = '.$nullDate.' OR publish_down >= '.$nowDate.')')
            ->where('state = 1');
        if(is_numeric($parent)){
            $query->where('parent = '.(int)$parent);
        }
        $query->order($order);
        
        return $query;
    }

    function getSavedProperties( $user_id )
    {
        $db         = JFactory::getDbo();
        
        // Filter by start and end dates.
        $nullDate   = $db->Quote($db->getNullDate());
        $date       = JFactory::getDate();
        $nowDate    = $db->Quote($date->toSql());
        
        $query = $db->getQuery(true);
        $query->select('*, p.title AS title, p.street AS street, s.id AS id, s.timestamp AS created, p.id as prop_id, s.id as id, p.alias as alias')
            ->from('#__iproperty_saved as s')
            ->leftJoin('#__iproperty as p on p.id = s.prop_id')
            ->where('s.user_id = '.(int)$user_id)
            ->where('s.active = 1 AND s.type = 0')
            ->where('(p.publish_up = '.$nullDate.' OR p.publish_up <= '.$nowDate.')')
            ->where('(p.publish_down = '.$nullDate.' OR p.publish_down >= '.$nowDate.')')
            ->order('timestamp DESC');
        
        return $query;
    }
    
    function getSavedSearches( $user_id )
    {
        $db         = JFactory::getDbo();
        
        $query = $db->getQuery(true);
        $query->select('*, s.notes AS title, s.id AS id, s.timestamp AS created')
            ->from('#__iproperty_saved as s')
            ->where('s.user_id = '.(int)$user_id)
            ->where('s.active = 1 AND s.type != 0')
            ->order('timestamp DESC');
        
        return $query;
    }    

    function deleteSavedProperty( $id, $user_id )
    {
        $db         = JFactory::getDbo();
        
        $query = $db->getQuery(true);
        $query->update('#__iproperty_saved')
            ->set('active = 0')
            ->where('id = '.(int)$id)
            ->where('user_id = '.(int)$user_id);
        
        return $query;
    }

    function saveProperty( $id, $user_id, $notes = '', $email_update = 1 )
    {
        $db = JFactory::getDBO();
        $query = $db->getQuery(true);
        $query->insert('#__iproperty_saved')
                ->columns('user_id, prop_id, notes, email_update, active')
                ->values((int)$user_id.', '.(int)$id.', '.$db->Quote($notes).', '.(int)$email_update.', 1');
        return $query;
    }

    function saveSearch( $searchstring, $user_id, $notes = '', $email_update = 1 )
    {
        $db = JFactory::getDBO();
        $query = $db->getQuery(true);
        $query->insert('#__iproperty_saved')
                ->columns('user_id, notes, search_string, email_update, active, type')
                ->values((int)$user_id.', '.$db->Quote($notes).', '.$db->Quote($searchstring).', '.(int)$email_update.', 1, 1');
        return $query;
    }
    
    function buildAgent($agent_id)
    {
        $db = JFactory::getDBO();
        
        $query = $db->getQuery(true);
        $query->select('a.*, c.id AS companyid, c.name AS companyname, CONCAT_WS(" ",fname,lname) AS name, a.alias as alias, c.alias as co_alias')
            ->from('#__iproperty_agents as a')
            ->leftJoin('#__iproperty_companies as c on c.id = a.company')
            ->where('a.id = '.(int)$agent_id)
            ->where('a.state = 1');
        
        $db->setQuery($query, 0, 1);
        return $db->loadObject();
    }

    function buildCompany($co_id)
    {
        $db = JFactory::getDBO();
        
        $query = $db->getQuery(true);
        $query->select('*')
            ->from('#__iproperty_companies')
            ->where('id = '.(int)$co_id)
            ->where('state = 1');
        
        $db->setQuery($query, 0, 1);
        return $db->loadObject();
    }
}
?>